﻿using System;
using System.Collections.Generic;
using System.Windows;
using System.Windows.Controls;
using System.Collections;
using System.Xml.Serialization;
using System.Xml;
using Telerik.Windows.Controls;
using Telerik.Windows.Data;
using System.Collections.ObjectModel;
using System.Windows.Threading;
using System.Windows.Media;

namespace Telerik.Examples.WP.JumpList
{
    public partial class FirstLook : ListExampleBase
    {
        private Button getMoreButton;
        private List<MovieViewModel> items;
        private ObservableCollection<MovieViewModel> source = new ObservableCollection<MovieViewModel>();
        private int lastLoadedIndex = 0;
        private DispatcherTimer timer;
        public FirstLook()
        {
            InitializeComponent();

			this.jumpList.FilterDescriptors.Add(new GenericFilterDescriptor<MovieViewModel>(movie => movie.Genre.Contains("Children")));
            this.radioAutomatic.IsChecked = true;

            this.timer = new DispatcherTimer();
            this.timer.Interval = TimeSpan.FromSeconds(2);
            this.timer.Tick += (object sender, EventArgs args) =>
            {
                this.LoadData();
                if (this.getMoreButton != null)
                {
                    this.getMoreButton.Content = "load more";
                    this.getMoreButton.IsEnabled = true;
                }
            };

            this.jumpList.GroupHeaderItemTap += this.jumpList_GroupHeaderItemTap;
            this.jumpList.DataRequested += this.jumpList_DataRequested;
            this.items = this.GetItems();
            this.LoadData();
        }

        private void OnGroupHeader_ItemTap(object sender, GroupHeaderItemTapEventArgs e)
        {
            Image navImage = e.OriginalSource as Image;
            if (navImage != null)
            {
                IDataSourceGroup dataGroup = this.jumpList.GetDataSourceItemForDataItem(e.Item.Content) as IDataSourceGroup;

                if (navImage.Tag != null && navImage.Tag.ToString().Equals("up"))
                {
                    int indexOfGroup = Array.IndexOf(this.jumpList.Groups, dataGroup.Value);
                    if (indexOfGroup > 0)
                    {
                        this.jumpList.BringIntoView(this.jumpList.Groups[indexOfGroup - 1]);
                        e.ShowGroupPicker = false;
                    }
                }
                else if (navImage.Tag != null && navImage.Tag.ToString().Equals("down"))
                {
                    int indexOfGroup = Array.IndexOf(this.jumpList.Groups, dataGroup.Value);
                    if (indexOfGroup < this.jumpList.Groups.Length - 1)
                    {
                        this.jumpList.BringIntoView(this.jumpList.Groups[indexOfGroup + 1]);
                        e.ShowGroupPicker = false;
                    }
                }
            }
        }

        protected override IEnumerable CreateItemsSource()
        {
            return this.source;
        }

        private void DisableVirtualization()
        {
            this.jumpList.DataVirtualizationMode = DataVirtualizationMode.None;
            this.radioAutomatic.IsEnabled = this.radioManual.IsEnabled = false;
            this.ListBox.ListFooterTemplate = this.LayoutRoot.Resources["ListFooterDataTemplate"] as DataTemplate;
        }

        private void LoadData()
        {
            this.timer.Stop();
            for (int i = 0; i < 10; i++)
            {
                this.source.Add(this.items[lastLoadedIndex++]);

                if (lastLoadedIndex == this.items.Count)
                {
                    this.DisableVirtualization();
                    break;
                }
            }
        }

        private void jumpList_DataRequested(object sender, EventArgs e)
        {
            if (!this.timer.IsEnabled)
            {
                this.timer.Start();
            }
        }

        private List<MovieViewModel> GetItems()
        {
            using (XmlReader xmlReader = XmlReader.Create("Data/NetFlixTop50.xml"))
            {
                int ranking = 1;
                XmlSerializer deserializer = new XmlSerializer(typeof(List<MovieViewModel>));
                List<MovieViewModel> movies = deserializer.Deserialize(xmlReader) as List<MovieViewModel>;

                foreach (MovieViewModel movie in movies)
                {
                    movie.Ranking = ranking++;
                }

                movies.Sort(
                    new Comparison<MovieViewModel>((MovieViewModel m1, MovieViewModel m2) =>
                    {
                        return m1.Genre.CompareTo(m2.Genre);
                    }));
                return movies;
            }
        }

        protected override RadDataBoundListBox ListBox
        {
            get
            {
                return this.jumpList;
            }
        }

        protected override string ExampleName
        {
            get
            {
                return "JumpList.FirstLook";
            }
        }

        private void jumpList_GroupHeaderItemTap(object sender, GroupHeaderItemTapEventArgs e)
        {
            Image image = e.ManipulationContainer as Image;
            if (image == null)
            {
                return;
            }

            if ((string)image.Tag == "up")
            {
                this.MoveUp(e);
            }
            else if ((string)image.Tag == "down")
            {
                this.MoveDown(e);
            }
        }

        private void MoveDown(GroupHeaderItemTapEventArgs e)
        {
            DataGroup[] groups = this.jumpList.Groups;
            int index = this.IndexOfGroup(groups, e.Item);
            index++;
            index = Math.Min(groups.Length - 1, index);
            this.jumpList.BringIntoView(groups[index]);

            e.ShowGroupPicker = false;
        }

        private void MoveUp(GroupHeaderItemTapEventArgs e)
        {
            DataGroup[] groups = this.jumpList.Groups;
            int index = this.IndexOfGroup(groups, e.Item);

            // find the first non-visible group on top of the tapped one
            while (true)
            {
                index--;
                if (index < 0)
                {
                    break;
                }

                if (!this.jumpList.IsItemInViewport(groups[index]))
                {
                    break;
                }
            }

            if (index >= 0)
            {
                this.jumpList.BringIntoView(groups[index]);
            }

            e.ShowGroupPicker = false;
        }

        private int IndexOfGroup(DataGroup[] groups, RadDataBoundListBoxItem item)
        {
            DataGroup hitGroup = item.Content as DataGroup;

            for (int i = 0; i < groups.Length; i++)
            {
                if (groups[i] == hitGroup)
                {
                    return i;
                }
            }

            return -1;
        }

        private void radioManual_Checked(object sender, RoutedEventArgs e)
        {
            this.jumpList.DataVirtualizationMode = DataVirtualizationMode.OnDemandManual;
        }

        private void radioAutomatic_Checked(object sender, RoutedEventArgs e)
        {
            this.jumpList.DataVirtualizationMode = DataVirtualizationMode.OnDemandAutomatic;
        }

        private void btnLoadMore_Click(object sender, RoutedEventArgs e)
        {
            this.getMoreButton = sender as Button;
            this.getMoreButton.Content = "loading...";
            this.getMoreButton.IsEnabled = false;
            if (!this.timer.IsEnabled)
            {
                this.timer.Start();
            }
        }

        private void btnResetClick_Click(object sender, RoutedEventArgs e)
        {
            this.ListBox.ListFooterTemplate = null;
            this.ListBox.ItemsSource = null;
            this.source.Clear();
            this.ListBox.ItemsSource = this.source;
            this.lastLoadedIndex = 0;
            this.radioAutomatic.IsEnabled = this.radioManual.IsEnabled = true;
            this.radioAutomatic.IsChecked = true;
            this.ListBox.DataVirtualizationMode = DataVirtualizationMode.OnDemandAutomatic;
            this.LoadData();
        }
    }
}
